#!/usr/bin/env python
# -*- coding: utf-8 -*-

import pyotherside

class Crasher:
    def __init__(self):
        pass

    def __del__(self):
        print('Finalizing', self)

def foo(arg):
    print('Got arg:', arg)
    return 'hello world'



#
# PyOtherSide: Asynchronous Python 3 Bindings for Qt 5 and Qt 6
# Copyright (c) 2011, 2013, Thomas Perl <m@thp.io>
#
# Permission to use, copy, modify, and/or distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
# INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
# LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
# OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
# PERFORMANCE OF THIS SOFTWARE.
#

import pyotherside
import random

COLORS = [(255, 255, 255, 255), (255, 0, 0, 0)]

for i in range(30):
    COLORS.insert(1, (
        255, # alpha
        random.randint(30, 255), # red
        random.randint(30, 255), # green
        random.randint(30, 255), # blue
    ))

# Of course, in production code, you don't want to calculate your mandelbrot
# set in CPython for performance reasons. This is just an example of what is
# possible with the image provider support in PyOtherSide.
def mandelbrot(x, y, steps):
    a = b = 0
    for i in range(steps):
        a, b = a**2 - b**2 + x, 2*a*b + y
        if a**2 + b**2 >= 4:
            break
    return i

def render_mandelbrot(image_id, requested_size):
    # when the url is: "image://python/xmin/xmax/ymin/ymax"
    # the image_id is: "xmin/xmax/ymin/ymax"

    parts = [float(x) for x in image_id.split('/')]
    x_range = parts[0:2]
    y_range = parts[2:4]

    # width and height will be -1 if not set in QML
    if requested_size == (-1, -1):
        requested_size = (200, 200)

    width, height = requested_size

    pixels = []
    for y in range(height):
        yy = y_range[0] + (y_range[1] - y_range[0]) * y / height
        for x in range(width):
            xx = x_range[0] + (x_range[1] - x_range[0]) * x / width
            pixels.extend(reversed(COLORS[mandelbrot(xx, yy, len(COLORS))]))
    return bytearray(pixels), (width, height), pyotherside.format_argb32

pyotherside.set_image_provider(render_mandelbrot)



import datetime

def submit_datetime(dt):
    assert dt is not None
    print('Python got datetime:', dt)
    return dt

def submit_date(date):
    assert date is not None
    print('Python got date:', date)
    return date

def submit_time(time):
    assert time is not None
    print('Python got time:', time)
    return time

def get_datetime_value():
    v = datetime.datetime.now()
    print('Python returning datetime:', v)
    return v

def get_time_value():
    v = datetime.datetime.now().time()
    print('Python returning time:', v)
    return v

def get_date_value():
    v = datetime.datetime.now().date()
    print('Python returning date:', v)
    return v



#
# PyOtherSide: Asynchronous Python 3 Bindings for Qt 5 and Qt 6
# Copyright (c) 2014, Thomas Perl <m@thp.io>
#
# Permission to use, copy, modify, and/or distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
# INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
# LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
# OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
# PERFORMANCE OF THIS SOFTWARE.
#

import sys
import pyotherside

from importlib import abc

class PyOtherSideQtRCImporter(abc.MetaPathFinder, abc.SourceLoader):
    def find_module(self, fullname, path):
        if path is None or all(x.startswith('qrc:') for x in path):
            if self.get_filename(fullname):
                return self

    def get_filename(self, fullname):
        basename = fullname.replace('.', '/')

        for import_path in sys.path:
            if not import_path.startswith('qrc:'):
                continue

            for candidate in ('{}/{}.py', '{}/{}/__init__.py'):
                filename = candidate.format(import_path, basename)
                if pyotherside.qrc_is_file(filename[len('qrc:'):]):
                    return filename

    def get_data(self, path):
        return pyotherside.qrc_get_file_contents(path[len('qrc:'):])

    def module_repr(self, m):
        return "<module '{}' from '{}'>".format(m.__name__, m.__file__)

sys.meta_path.append(PyOtherSideQtRCImporter())



# A very simple notetaking application that uses Python to load
# and save a string in a text file in the user's home directory.

import os
import threading
import time

import pyotherside

class Notes:
    def __init__(self):
        self.filename = os.path.expanduser('~/pyotherside_notes.txt')
        self.thread = None
        self.new_text = self.get_contents()

    def save_now(self):
        print('Saving file right away at exit')
        self._update_file(now=True)

    def _update_file(self, now=False):
        if not now:
            time.sleep(3)
        print('Saving file now')
        open(self.filename, 'w').write(self.new_text)
        self.thread = None

    def get_contents(self):
        if os.path.exists(self.filename):
            return open(self.filename).read()
        else:
            return '<new file>'

    def set_contents(self, text):
        self.new_text = text
        if self.thread is None:
            print('Scheduling saving of file')
            self.thread = threading.Thread(target=self._update_file)
            self.thread.start()

notes = Notes()
pyotherside.atexit(notes.save_now)



# -*- coding: utf-8 -*-

class Foo(object):
    def __init__(self, name):
        print('new Foo(', name, ')')
        self.name = name
        self.bar = 4711

    def methodman(self, something):
        return 'I came to bring {}'.format(something)

    def __del__(self):
        print('__del__ called on', self.name)


def get_foo():
    print('get_foo()')
    return Foo('Hello World!')

def set_foo(foo):
    print('set_foo(', foo, ')')
    return foo.name

